@@ -12,16 +12,21 @@ import sys
 from os import getenv
 from os.path import expanduser, isfile
 
-from .constants import BACKEND_URL, INGESTION_BACKEND_URL, LEGACY_BACKEND_URL
+from .constants import (
+    BACKEND_URL,
+    INGESTION_BACKEND_URL,
+    LEGACY_BACKEND_URL,
+    SERVERLESS_EXTENSION_URL,
+)
 from .utils import HAS_TYPING
 
 if sys.version_info[0] < 3:
-    from ConfigParser import RawConfigParser, Error as ConfigError
+    from ConfigParser import Error as ConfigError, RawConfigParser
 else:
-    from configparser import RawConfigParser, Error as ConfigError
+    from configparser import Error as ConfigError, RawConfigParser
 
 if HAS_TYPING:
-    from typing import Optional, Any, Mapping
+    from typing import Any, Mapping, Optional
 
 LOGGER = logging.getLogger(__name__)
 
@@ -67,11 +72,11 @@ CONFIG_DEFAULT_VALUE = {
     # Enable PII scrubbing (password, ...)
     "STRIP_SENSITIVE_DATA": True,
     # Sensitive keys to strip
-    "STRIP_SENSITIVE_KEYS": "password, secret, passwd, authorization, api_key, apikey, access_token",
+    "STRIP_SENSITIVE_KEYS": "password, secret, passwd, authorization, api_key, apikey, access_token, token",
     # Sensitive regexp (matches items like credit cards)
     "STRIP_SENSITIVE_REGEX": r"^(?:\d[ -]*?){13,16}$",
     # Do not block longer than this period in seconds during startup
-    "INSTRUMENTATION_TIMEOUT": 6.0,
+    "INSTRUMENTATION_TIMEOUT": 10.0,
     # Graceful timeout, max time waiting for the runner thread to exit before exiting
     "GRACEFUL_TIMEOUT": 5.0,
     # Static instrumentation, enabled by default
@@ -81,6 +86,10 @@ CONFIG_DEFAULT_VALUE = {
     # Max length of Sqreen queues
     "MAX_QUEUE_LENGTH": 100,
     "MAX_OBS_QUEUE_LENGTH": 10000,
+    # Serverless configuration
+    "SERVERLESS_DETECTION": True,
+    "SERVERLESS_EXTENSION_URL": SERVERLESS_EXTENSION_URL,
+    "SERVERLESS_INSTRUMENTATION_TIMEOUT": 2.0,
 }
 
 
