@@ -128,6 +128,8 @@ class BatchDeliverer(SimpleDeliverer):
             result.append("rex-{}".format(exc_dict["klass"]))
         for sdk_entry in observed_dict.get("sdk", []):
             result.append("sdk-{}".format(sdk_entry[0]))
+        for signal_entry in observed_dict.get("signals", []):
+            result.append("signal-{}".format(signal_entry.get("signal_name")))
         return result
 
     @classmethod
