@@ -6,8 +6,10 @@
 #
 
 from .aiohttp import AioHTTPFrameworkAdapter
+from .aws_chalice import AWSChaliceFrameworkAdapter
 from .aws_lambda import AWSLambdaFrameworkAdapter
 from .django import DjangoFrameworkAdapter
+from .falcon import FalconFrameworkAdapter
 from .flask import FlaskFrameworkAdapter
 from .pyramid import PyramidFrameworkAdapter
 
@@ -20,3 +22,5 @@ def init(interface_manager):
     interface_manager.register(FlaskFrameworkAdapter())
     interface_manager.register(PyramidFrameworkAdapter())
     interface_manager.register(AWSLambdaFrameworkAdapter())
+    interface_manager.register(AWSChaliceFrameworkAdapter())
+    interface_manager.register(FalconFrameworkAdapter())
