@@ -5,12 +5,12 @@
 #     https://www.sqreen.io/terms.html
 #
 from ....rules import RuleCallback
-from ....workflow import Transport
+from ....rules_actions import RecordTransportMixin
 
 
-class Psycopg2TransportCallback(RuleCallback):
+class Psycopg2TransportCallback(RecordTransportMixin, RuleCallback):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         try:
             # TODO parse args[0] if provided
             host = kwargs.get("host", "127.0.0.1")
@@ -18,17 +18,15 @@ class Psycopg2TransportCallback(RuleCallback):
         except Exception:
             host, port = None, None
 
-        transport = Transport({
+        transport = {
             "type": "postgres",
             "host": host,
             "host_port": port,
-        })
-
-        context = self.runner.interface_manager.call("context")
-        self.runner.interface_manager.call("new_data_out_pre", context, transport)
+        }
+        self.record_transport("client", transport)
 
 
-class Psycopg2TransportAdapter:
+class Psycopg2TransportAdapter(object):
 
     def instrumentation_callbacks(self, runner, storage):
         return [
