@@ -5,29 +5,27 @@
 #     https://www.sqreen.io/terms.html
 #
 from ....rules import RuleCallback
-from ....workflow import Transport
+from ....rules_actions import RecordTransportMixin
 
 
-class RedisTransportCallback(RuleCallback):
+class RedisTransportCallback(RecordTransportMixin, RuleCallback):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         try:
             host = kwargs["host"]
             port = kwargs["port"]
         except Exception:
             host, port = None, None
 
-        transport = Transport({
+        transport = {
             "type": "redis",
             "host": host,
             "host_port": port,
-        })
-
-        context = self.runner.interface_manager.call("context")
-        self.runner.interface_manager.call("new_data_out_pre", context, transport)
+        }
+        self.record_transport("client", transport)
 
 
-class RedisTransportAdapter:
+class RedisTransportAdapter(object):
 
     def instrumentation_callbacks(self, runner, storage):
         return [
