@@ -10,13 +10,11 @@ import logging
 import sys
 
 from .._vendors.wrapt import FunctionWrapper
-from ..constants import ACTIONS
 from ..exceptions import ActionBlock, ActionRedirect, AttackBlocked
 from .hook_point import (
     execute_failing_callbacks,
     execute_post_callbacks,
     execute_pre_callbacks,
-    valid_args,
 )
 
 LOGGER = logging.getLogger(__name__)
@@ -36,32 +34,36 @@ def async_hook_point(strategy, hook_name, hook_method, original):
         )
         strategy.before_hook_point()
         key = (hook_name, hook_method)
+        callbacks = strategy.settings.get_callbacks(key)
+        options = {}
 
         # Call pre callbacks.
-        action = execute_pre_callbacks(key, strategy, instance, args, kwargs)
+        action = execute_pre_callbacks(key, callbacks["pre"], instance, args, kwargs, options)
 
-        if action.get("status") == ACTIONS["RAISE"]:
+        status = action.get("status")
+        if status is None:
+            pass
+        elif status == "modify_args":
+            args, kwargs = action["args"]
+        elif status == "override":
+            return action.get("new_return_value")
+        elif status == "raise":
             LOGGER.debug(
                 "Callback %s detected an attack", action.get("rule_name")
             )
             raise AttackBlocked(action.get("rule_name"))
-        elif action.get("status") == ACTIONS["ACTION_BLOCK"]:
+        elif status == "action_block":
             LOGGER.debug(
                 "Action %s blocked the request", action.get("action_id")
             )
             raise ActionBlock(action.get("action_id"))
-        elif action.get("status") == ACTIONS["ACTION_REDIRECT"]:
+        elif status == "action_redirect":
             LOGGER.debug(
                 "Action %s redirected the request to %r",
                 action.get("action_id"),
                 action["target_url"],
             )
             raise ActionRedirect(action.get("action_id"), action["target_url"])
-        elif action.get("status") == ACTIONS["OVERRIDE"]:
-            return action.get("new_return_value")
-        elif action.get("status") == ACTIONS["MODIFY_ARGS"]:
-            if valid_args(action["args"]):
-                args, kwargs = action["args"]
 
         # Call the original method.
         retry = True
@@ -71,24 +73,25 @@ def async_hook_point(strategy, hook_name, hook_method, original):
                 # Try to call the original coroutine.
                 result = yield from wrapped(*args, **kwargs)
             except Exception:
-                # In case of error, call fail callbacks with exception infos.
-                exc_info = sys.exc_info()
-
                 # Either raise an exception, set a return value or retry.
+                options["exc_info"] = sys.exc_info()
                 action = execute_failing_callbacks(
-                    key, strategy, instance, exc_info, args, kwargs
+                    key, callbacks["failing"], instance, args, kwargs, options
                 )
 
-                if action.get("status") == ACTIONS["RAISE"]:
+                status = action.get("status")
+                if status is None:
+                    pass
+                elif status == "override":
+                    return action.get("new_return_value")
+                elif status == "retry":
+                    retry = True
+                elif status == "raise":
                     LOGGER.debug(
                         "Callback %s detected an attack",
                         action.get("rule_name"),
                     )
                     raise AttackBlocked(action.get("rule_name"))
-                elif action.get("status") == ACTIONS["RETRY"]:
-                    retry = True
-                elif action.get("status") == ACTIONS["OVERRIDE"]:
-                    return action.get("new_return_value")
 
                 # Be sure to raise if no retry or override.
                 if retry is False:
@@ -96,17 +99,21 @@ def async_hook_point(strategy, hook_name, hook_method, original):
 
         # Then call post callback in reverse order to simulate decorator
         # behavior.
+        options["result"] = result
         action = execute_post_callbacks(
-            key, strategy, instance, result, args, kwargs
+            key, callbacks["post"], instance, args, kwargs, options
         )
 
-        if action.get("status") == ACTIONS["RAISE"]:
+        status = action.get("status")
+        if status is None:
+            pass
+        elif status == "override":
+            return action.get("new_return_value")
+        elif status == "raise":
             LOGGER.debug(
                 "Callback %s detected an attack", action.get("rule_name")
             )
             raise AttackBlocked(action.get("rule_name"))
-        elif action.get("status") == ACTIONS["OVERRIDE"]:
-            return action.get("new_return_value")
 
         # Return the original value.
         return result
