@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -8,12 +8,13 @@
 """
 import logging
 import sys
+import threading
 
-try:
-    from importlib import find_loader
+if sys.version_info >= (3, 4):
+    from importlib.util import find_spec
     PY3_IMPORT_HOOK = True
-except ImportError:
-    find_loader = None
+else:
+    find_spec = None
     PY3_IMPORT_HOOK = False
 
 LOGGER = logging.getLogger(__name__)
@@ -24,7 +25,7 @@ def get_hook_parent(module, hook_class):
     and returns both the hook parent and the full hook path
     """
     if hook_class:
-        if '.' in hook_class:
+        if "." in hook_class:
             raise NotImplementedError()
         else:
             return getattr(module, hook_class, None)
@@ -40,11 +41,11 @@ def get_module_parent(module_name):
     >>> get_module_parent('django')
     'django'
     """
-    splitted = module_name.split('.')
+    splitted = module_name.split(".")
     if len(splitted) == 1:
         return module_name
     else:
-        return '.'.join(splitted[:-1])
+        return ".".join(splitted[:-1])
 
 
 def get_original(module, module_class_name, module_object_name):
@@ -67,9 +68,13 @@ def get_original(module, module_class_name, module_object_name):
     module_object = get_hook_parent(module_class, module_object_name)
 
     if module_object is None:
-        LOGGER.info("Module class %r::%r (%r) has no attribute %r",
-                    module, module_class_name, module_class,
-                    module_object_name)
+        LOGGER.info(
+            "Module class %r::%r (%r) has no attribute %r",
+            module,
+            module_class_name,
+            module_class,
+            module_object_name,
+        )
         return None
 
     return module_object
@@ -87,6 +92,7 @@ def update_original(module, patched, module_class_name, module_object_name):
 class ExistingPatchOnSameObject(Exception):
     """ Raise in case a patcher is already present for the same object
     """
+
     pass
 
 
@@ -98,8 +104,16 @@ class ModuleFinder(object):
     def __init__(self):
         self.patchers = {}
         self.loading = {}
+        # The ModuleFinder is used across all threads so we must lock the
+        # patching operation to prevent race conditions. In fact, a thread can be
+        # loading a module while a second tries to patch it. It leads to random errors
+        # such as "module has no attribute".
+        # The lock is recursive because the find_module method calls itself.
+        self.lock = threading.RLock()
 
-    def register_patcher(self, module_name, module_class, module_object, patcher):
+    def register_patcher(
+        self, module_name, module_class, module_object, patcher
+    ):
         """ Register a patcher for a specific path, a path is composed of
         module_name, module_class and module_object.
         For example: 'itertools:None:count' would target the function named count
@@ -113,13 +127,15 @@ class ModuleFinder(object):
         raised.
         """
         path = "%s:%s:%s" % (module_name, module_class, module_object)
-        LOGGER.debug("Adding patcher for path %s %r", path, patcher)
-        patchers = self.patchers.setdefault(module_name, {}).setdefault(module_class, {})
+        LOGGER.debug("Adding patcher on %s", path)
+        patchers = self.patchers.setdefault(module_name, {}).setdefault(
+            module_class, {}
+        )
 
         if module_object in patchers:
-            raise ExistingPatchOnSameObject('Patcher already here')
+            raise ExistingPatchOnSameObject("Patcher already here")
 
-        patchers[module_object] = {'patcher': patcher, 'patched': False}
+        patchers[module_object] = {"patcher": patcher, "patched": False}
 
     def inject(self):
         """ Inject the global import hook in sys.meta_path if it's not already
@@ -132,25 +148,32 @@ class ModuleFinder(object):
         """ Try to apply not already applied patchers on modules already present
         in sys.modules as they would not be applied otherwise.
         """
-        for module_name, module_classes in self.patchers.items():
+        # Block the patching operation if there are still modules loading.
+        with self.lock:
+            for module_name, _ in self.patchers.items():
 
-            if module_name in sys.modules:
-                msg = "Module %s was already imported, hooking now"
-                LOGGER.warning(msg, module_name)
+                if module_name in sys.modules:
+                    msg = "Module %s was already imported, hooking now"
+                    LOGGER.warning(msg, module_name)
 
-                self.patch_module(module_name, sys.modules[module_name])
+                    self.patch_module(module_name, sys.modules[module_name])
 
     def patch_module(self, fullname, module):
         """ For a module named fullname, apply all patchers defined for it
         """
-        for module_class, module_objects in self.patchers.get(fullname, {}).items():
+        for module_class, module_objects in self.patchers.get(
+            fullname, {}
+        ).items():
             for module_object, patcher in module_objects.items():
-                if patcher['patched'] is False:
+                if patcher["patched"] is False:
                     self._patch(module, module_class, module_object, patcher)
                 else:
                     path = "%s:%s:%s" % (fullname, module_class, module_object)
-                    LOGGER.debug('Patcher %r already applied for %s',
-                                 patcher['patcher'], path)
+                    LOGGER.debug(
+                        "Patcher %r already applied for %s",
+                        patcher["patcher"],
+                        path,
+                    )
 
         return module
 
@@ -161,11 +184,11 @@ class ModuleFinder(object):
         original = get_original(module, module_class, module_object)
         if original is not None:
             # Patch it
-            patched = patcher['patcher'](original)
+            patched = patcher["patcher"](original)
             # And update back the module
             update_original(module, patched, module_class, module_object)
         # Mark as already patched
-        patcher['patched'] = True
+        patcher["patched"] = True
 
     def find_module(self, fullname, path=None):
         """ Import hook main API, called when the application try to import
@@ -175,55 +198,56 @@ class ModuleFinder(object):
         if fullname not in self.patchers:
             return
 
-        is_loading = fullname in self.loading
-        LOGGER.info('Finding module %s, loading? %s, %r', fullname, is_loading, sys.meta_path)
-        # Avoid recursion as we import the module in a way that recheck every
-        # import hook
-        if is_loading:
-            return None
+        # Lock because we are currently loading a module.
+        with self.lock:
+            is_loading = fullname in self.loading
+            # Avoid recursion as we import the module in a way that recheck every
+            # import hook
+            if is_loading:
+                LOGGER.debug("Looking for already loading module %s", fullname)
+                return
 
-        self.loading[fullname] = True
-        try:
+            self.loading[fullname] = True
+            try:
 
-            # In python 2, call back the whole import mechanism but ignoring
-            # ourself
-            if PY3_IMPORT_HOOK is False:
-                __import__(fullname)
+                # In python 2, call back the whole import mechanism but ignoring
+                # ourself
+                if PY3_IMPORT_HOOK is False:
+                    __import__(fullname)
 
-                modules_keys = sys.modules
-                if fullname in modules_keys or get_module_parent(fullname) in modules_keys:
-                    return self
+                    modules_keys = sys.modules
+                    if (
+                        fullname in modules_keys
+                        or get_module_parent(fullname) in modules_keys
+                    ):
+                        return self
+                # In python 3, call loader
                 else:
-                    LOGGER.warning("Error, module not loaded")
-            # In python 3, call loader
-            else:
-                loader = find_loader(fullname, path)
+                    spec = find_spec(fullname, path)
+                    if spec is not None and spec.loader is not None:
+                        return ModuleLoader(spec.loader, self)
 
-                # Maybe this module is not found
-                if loader is None:
-                    return None
+                LOGGER.debug("Module %s was not loaded", fullname)
 
-                return ModuleLoader(loader, self)
-
-        finally:
-            del self.loading[fullname]
-
-        return None
+            finally:
+                del self.loading[fullname]
 
     def load_module(self, fullname):
         """ Load the module either from sys.modules or from its parent module.
         Used only in Python 2.
         """
-        if fullname in sys.modules:
-            module = sys.modules[fullname]
-        else:
-            parent_module_name = get_module_parent(fullname)
-            if parent_module_name in sys.modules:
-                parent_module = sys.modules[parent_module_name]
+        # Block the patching while we are loading a module.
+        with self.lock:
+            if fullname in sys.modules:
+                module = sys.modules[fullname]
+            else:
+                parent_module_name = get_module_parent(fullname)
+                if parent_module_name in sys.modules:
+                    parent_module = sys.modules[parent_module_name]
 
-                module = getattr(parent_module, fullname.split('.')[-1])
+                    module = getattr(parent_module, fullname.split(".")[-1])
 
-        return self.patch_module(fullname, module)
+            return self.patch_module(fullname, module)
 
 
 class ModuleLoader(object):
@@ -237,5 +261,7 @@ class ModuleLoader(object):
     def load_module(self, fullname):
         """ Import hook in python 3 call for loading a module
         """
-        module = self.loader.load_module(fullname)
-        return self.finder.patch_module(fullname, module)
+        # Block the ModuleFinder from patching while we are loading a module.
+        with self.finder.lock:
+            module = self.loader.load_module(fullname)
+            return self.finder.patch_module(fullname, module)
