@@ -16,14 +16,14 @@ class AWSLambdaMiddleware(BaseMiddleware):
     def wrapper(self, wrapped, instance, args, kwargs):
         def _extract(event, context, *args, **kwargs):
             return event, context
-
-        self.execute_pre_callbacks(args=_extract(*args, **kwargs))
+        transaction = self.create_transaction()
+        self.execute_pre_callbacks(transaction, _extract(*args, **kwargs))
         try:
             ret = wrapped(*args, **kwargs)
         except Exception as e:
-            ret = self.execute_failing_callbacks(e)
+            ret = self.execute_failing_callbacks(transaction, e)
             if ret:
                 return ret
             raise
         else:
-            return self.execute_post_callbacks(ret)
+            return self.execute_post_callbacks(transaction, ret)
