@@ -21,9 +21,10 @@ class DjangoMiddleware(BaseMiddleware):
         (None, request, view_func, view_args, view_kwargs)
         """
         self.strategy.before_hook_point()
+        transaction = request.__sqreen_transaction = self.create_transaction()
         args = (request, view_func, view_args, view_kwargs)
 
-        self.execute_pre_callbacks(args)
+        self.execute_pre_callbacks(transaction, args)
 
     def process_response(self, request, response):
         """ Call wrapped_callback.post, raise AttackBlock if needed or returns
@@ -34,22 +35,29 @@ class DjangoMiddleware(BaseMiddleware):
         # Record request if we don't have one yet
         runtime.store_request_default(DjangoRequest(request))
 
+        transaction = getattr(request, "__sqreen_transaction", None)
+        if transaction is None:
+            transaction = request.__sqreen_transaction = self.create_transaction()
+
         # Execute pre_callbacks if the response is a 404 because the callback
         # process_view was not called by Django
         if response.status_code == 404:
             args = (request, None, None, None)
-            self.execute_pre_callbacks(args)
+            self.execute_pre_callbacks(transaction, args)
         args = (request,)
-        return self.execute_post_callbacks(response, args)
+        return self.execute_post_callbacks(transaction, response, args)
 
     def process_exception(self, request, exception):
         """ Call wrapped_callback.failing, always return None.
         Django failing callbacks will receive these arguments:
         (None, exception, request)
         """
+        transaction = getattr(request, "__sqreen_transaction", None)
+        if transaction is None:
+            transaction = request.__sqreen_transaction = self.create_transaction()
         args = (request,)
 
         # Record request if we don't have one yet
         runtime.store_request_default(DjangoRequest(request))
 
-        self.execute_failing_callbacks(exception, args)
+        self.execute_failing_callbacks(transaction, exception, args)
