@@ -23,16 +23,17 @@ class FlaskMiddleware(BaseMiddleware):
             (None, exception)
             """
             self.strategy.before_hook_point()
+            transaction = self.create_transaction()
 
-            self.execute_pre_callbacks(record_attack=True)
+            self.execute_pre_callbacks(transaction, (), record_attack=True)
 
             try:
                 response = wrapped(*args, **kwargs)
             except Exception:
-                self.execute_failing_callbacks(sys.exc_info())
+                self.execute_failing_callbacks(transaction, sys.exc_info())
                 raise
             return self.execute_post_callbacks(
-                response, record_attack=True
+                transaction, response, record_attack=True
             )
 
         return FunctionWrapper(original, wrapper)
