@@ -27,20 +27,21 @@ class PyramidMiddleware(BaseMiddleware):
 
             request = args[0]
             self.strategy.before_hook_point()
+            transaction = self.create_transaction()
             pre_args = (request,)
-            self.execute_pre_callbacks(pre_args, record_attack=True)
+            self.execute_pre_callbacks(transaction, pre_args, record_attack=True)
 
             try:
                 response = wrapped(*args, **kwargs)
             except Exception as e:
                 if isinstance(e, Response):
-                    self.execute_post_callbacks(e)
+                    self.execute_post_callbacks(transaction, e)
                 else:
-                    self.execute_failing_callbacks(sys.exc_info())
+                    self.execute_failing_callbacks(transaction, sys.exc_info())
                 raise
 
             return self.execute_post_callbacks(
-                response, record_attack=True
+                transaction, response, record_attack=True
             )
 
         return FunctionWrapper(original, wrapper)
