@@ -8,6 +8,7 @@
 """
 
 from ...utils import HAS_ASYNCIO
+from .aws_lambda import AWSLambdaStrategy
 from .base import BaseStrategy
 from .dbapi2 import DBApi2Strategy
 from .django_strategy import DjangoStrategy
@@ -28,6 +29,7 @@ if HAS_ASYNCIO:
     from .async_import_hook import AsyncImportHookStrategy
 
 __all__ = [
+    "AWSLambdaStrategy",
     "AioHTTPHookStrategy",
     "AioHTTPInstallStrategy",
     "AsyncEventLoopStrategy",
