@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -8,7 +8,7 @@
 
 import logging
 
-from ...async_context import create_task_factory
+from ..._vendors.wrapt import FunctionWrapper
 from .import_hook import ImportHookStrategy
 
 LOGGER = logging.getLogger(__name__)
@@ -22,16 +22,21 @@ def _patch_loop_cls(loop_cls):
     orig_init = loop_cls.__init__
     orig_set_task_factory = loop_cls.set_task_factory
 
-    def __init__(self, *args, **kwargs):
-        orig_init(self, *args, **kwargs)
-        self.set_task_factory(None)
+    def __init__wrapper(wrapped, instance, args, kwargs):
+        wrapped(*args, **kwargs)
+        instance.set_task_factory(None)
 
-    def set_task_factory(self, factory):
-        factory = create_task_factory(factory)
-        orig_set_task_factory(self, factory)
+    def set_task_factory_wrapper(wrapped, instance, args, kwargs):
+        from ...async_context import create_task_factory
 
-    setattr(loop_cls, '__init__', __init__)
-    setattr(loop_cls, 'set_task_factory', set_task_factory)
+        new_args = list(args)
+        new_args[0] = create_task_factory(args[0])
+        wrapped(*new_args, **kwargs)
+
+    setattr(loop_cls, "__init__",
+            FunctionWrapper(orig_init, __init__wrapper))
+    setattr(loop_cls, "set_task_factory",
+            FunctionWrapper(orig_set_task_factory, set_task_factory_wrapper))
     return loop_cls
 
 
