@@ -203,7 +203,7 @@ def custom_connect(inner_wrapper, module_path, original_connect):
         _self_sqreen_wrapper = connection_wrapper
 
     def wrapper(wrapped, instance, args, kwargs):
-        conn = wrapped(*args, **kwargs)
+        conn = inner_wrapper(module_path, "connect")(wrapped, None, args, kwargs)
         if hasattr(conn, "execute"):
             # SQLite 3 has some extensions to DBApi2
             return CustomConnectionExtended(conn)
@@ -236,36 +236,12 @@ class DBApi2Strategy(BaseStrategy):
     'psycopg2::Cursor' will correctly reference all psycopg2 cursor methods.
     """
 
-    def __init__(
-        self,
-        module_path,
-        observation_queue,
-        queue,
-        import_hook,
-        before_hook_point=None,
-    ):
-        super(DBApi2Strategy, self).__init__(
-            observation_queue, queue, import_hook, before_hook_point
+    def __init__(self, *args, **kwargs):
+        super(DBApi2Strategy, self).__init__(*args, **kwargs)
+        self.module_path = self.strategy_id
+        self.import_hook.register_patcher(
+            self.module_path, None, "connect", self.import_hook_callback
         )
-        self.module_path = module_path
-
-        self.hooked = False
-
-    def hook(self):
-        """ Accept a callback and store it. If it's the first callback
-        for this strategy, actually hook to the endpoint.
-
-        Once hooked, it will instrument all method on connection and cursor.
-        But if no callback is defined on a specific method, the overhead will
-        be minimal.
-        """
-        # Check if we already hooked at
-        if not self.hooked:
-            self.import_hook.register_patcher(
-                self.module_path, None, "connect", self.import_hook_callback
-            )
-
-            self.hooked = True
 
     def import_hook_callback(self, original):
         """ Monkey-patch the object located at hook_class.hook_name on an
@@ -285,8 +261,3 @@ class DBApi2Strategy(BaseStrategy):
             return callback.hook_module.split("::", 1)[0]
         else:
             return callback.hook_module
-
-    def _restore(self):
-        """ The hooked module will always stay hooked
-        """
-        pass
