@@ -30,21 +30,12 @@ class FlaskStrategy(FrameworkStrategy):
     HOOK_CLASS = "Flask"
     HOOK_METHOD = "full_dispatch_request"
 
-    def __init__(
-        self,
-        strategy_key,
-        observation_queue,
-        queue,
-        import_hook,
-        before_hook_point=None,
-    ):
-        super(FlaskStrategy, self).__init__(
-            strategy_key,
-            observation_queue,
-            queue,
-            import_hook,
-            before_hook_point,
-        )
+    def __init__(self, *args, **kwargs):
+        super(FlaskStrategy, self).__init__(*args, **kwargs)
 
-        self.middleware = FlaskMiddleware(self, observation_queue, queue)
+        self.middleware = FlaskMiddleware(self)
         self.wrapper = replace_wsgi_app
+
+    @classmethod
+    def get_early_strategy_id(cls):
+        return None
