@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -9,14 +9,13 @@
 
 from logging import getLogger
 
-from ..middlewares import PyramidMiddleware
+from ..middlewares.pyramid_middleware import PyramidMiddleware
 from .framework import FrameworkStrategy
 
 LOGGER = getLogger(__name__)
 
 
 def tween_call_insert(original, middleware):
-
     def wrapped_tween_call(self, handler, *args, **kwargs):
         LOGGER.debug("Execute tween_call_insert")
         new_handler = middleware(handler)
@@ -29,7 +28,7 @@ def tween_call_insert(original, middleware):
 class PyramidStrategy(FrameworkStrategy):
     """ Strategy for Pyramid peripheric callbacks.
 
-    It injects a custom DjangoFramework that calls callbacks for each
+    It injects a custom PyramidMiddleware that calls callbacks for each
     lifecycle method
     """
 
@@ -37,9 +36,11 @@ class PyramidStrategy(FrameworkStrategy):
     HOOK_CLASS = "Tweens"
     HOOK_METHOD = "__call__"
 
-    def __init__(self, strategy_key, observation_queue, queue, import_hook, before_hook_point=None):
-        super(PyramidStrategy, self).__init__(strategy_key, observation_queue, queue,
-                                              import_hook, before_hook_point)
-
-        self.middleware = PyramidMiddleware(self, observation_queue, queue)
+    def __init__(self, *args, **kwargs):
+        super(PyramidStrategy, self).__init__(*args, **kwargs)
+        self.middleware = PyramidMiddleware(self)
         self.wrapper = tween_call_insert
+
+    @classmethod
+    def get_early_strategy_id(cls):
+        return None
