@@ -17,7 +17,7 @@ LOG_FORMAT = (
 )
 
 
-def configure_root_logger(log_level, log_location=None, root=ROOT_LOGGER_NAME):
+def configure_root_logger(log_level=logging.CRITICAL, log_location=None, root=ROOT_LOGGER_NAME):
     """ Configure the sqreen root logger. Set following settings:
 
     - log_level
@@ -31,35 +31,31 @@ def configure_root_logger(log_level, log_location=None, root=ROOT_LOGGER_NAME):
 
     # Don't propagate messages to upper loggers
     logger.propagate = False
+    logger.handlers = []
 
     formatter = logging.Formatter(LOG_FORMAT)
 
-    handlers = []
-
     # Configure the stderr handler configured on CRITICAL level
     stderr_handler = logging.StreamHandler()
     stderr_handler.setFormatter(formatter)
-    handlers.append(stderr_handler)
+    logger.addHandler(stderr_handler)
+
+    try:
+        logger.setLevel(log_level)
+    except ValueError:
+        logger.error("Unknown log_level %r, default log level is CRITICAL", log_level)
+        logger.setLevel(logging.CRITICAL)
 
     if log_location is not None:
         try:
             filehandler = logging.FileHandler(log_location)
             os.chmod(log_location, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP)
             filehandler.setFormatter(formatter)
-            handlers.append(filehandler)
+            logger.addHandler(filehandler)
         except (OSError, IOError):
             msg = "Couldn't use %s as sqreen log location, fallback to stderr."
-            logger.exception(msg, log_location)
-
-    if logger.handlers:
-        logger.handlers = []
-
-    for handler in handlers:
-        logger.addHandler(handler)
-
-    try:
-        logger.setLevel(log_level)
-    except ValueError:
-        logger.error("Unknown log_level %r, don't alter log level", log_level)
+            logger.error(msg, log_location, exc_info=True)
+        else:
+            stderr_handler.setLevel(logging.CRITICAL)
 
     return logger
