@@ -10,10 +10,10 @@ import logging
 from functools import wraps
 
 from .condition_evaluator import ConditionEvaluator, is_condition_empty
-from .constants import LIFECYCLE_METHODS
 from .remote_exception import raw_traceback_formatter, traceback_formatter
 from .runtime_storage import runtime
-from .utils import now
+from .samplers import ScopedTracingSampler
+from .utils import get_datadog_correlation_ids, now
 
 LOGGER = logging.getLogger(__name__)
 
@@ -22,7 +22,7 @@ class BaseCallback(object):
     """ Base class for callbacks.
     """
 
-    SECTIONS = ("request", "response", "params", "headers", "context")
+    DEFAULT_EXCEPTION_SECTIONS = ("request", "response", "params", "headers", "context")
 
     SUPPORTS_BUDGET = False  # override constants in subclasses supporting it
     INTERRUPTIBLE = True
@@ -40,6 +40,7 @@ class BaseCallback(object):
         payload_sections=None,
         priority=None,
         attack_type=None,
+        sampling=None,
         storage=None,
     ):
         self.rule_name = rule_name
@@ -51,13 +52,11 @@ class BaseCallback(object):
         self.block = block or False
         self.test = test or False
         self.metrics = metrics or []
+        self.sampling = sampling
         self.storage = storage or runtime
 
         self.attack_type = attack_type
 
-        # Payload sections.
-        if payload_sections is None:
-            payload_sections = self.SECTIONS
         self.payload_sections = payload_sections
 
         # Priority (lower number means higher priority)
@@ -88,6 +87,18 @@ class BaseCallback(object):
             return self.runner.performance_metrics and self.runner.performance_metrics_settings.enabled()
         return False
 
+    def action_processor(self, result_action, options):
+        """ Return the callback method results.
+        """
+        return result_action
+
+    def _apply_action_processor(self, methods):
+        for lifecycle in methods:
+            lifecycle_method = getattr(self, lifecycle, None)
+            if lifecycle_method is not None:
+                wrapped = action_processor_wrapper(lifecycle_method, self)
+                setattr(self, lifecycle, wrapped)
+
     def _apply_debug_log(self, methods):
         if not LOGGER.isEnabledFor(logging.DEBUG):
             return
@@ -111,13 +122,34 @@ class BaseCallback(object):
                     lifecycle_method, self.storage, metric_name)
                 setattr(self, lifecycle, wrapped)
 
-    def get_remaining_budget(self, callback_kwargs=None):
+    def _apply_sampling(self, methods):
+        """ Wrap each lifecycle methods if the Rule define them and if we have
+        sampling on the rule.
+        """
+        sampling = self.sampling
+        if sampling is None:
+            return
+
+        # XXX this is to circumvent a problem with rules defaulting to an empty sampling list
+        # instead of using null.
+        if not sampling:
+            return
+
+        sampler = ScopedTracingSampler({"enabled": True, "sampling": sampling})
+        for lifecycle in methods:
+            lifecycle_method = getattr(self, lifecycle, None)
+            if lifecycle_method is not None:
+                wrapped = sampling_wrapper(
+                    lifecycle_method, self, sampler, lifecycle)
+                setattr(self, lifecycle, wrapped)
+
+    def get_remaining_budget(self, callback_options=None):
         """ Get the remaining time we have for running this callback according
         to the request budget if there is one. The budget can be overriden
-        with a special option in kwargs. Return None otherwise.
+        with a special option in options. Return None otherwise.
         """
-        if callback_kwargs is not None:
-            override_budget = callback_kwargs.pop("__sqreen_override_budget", None)
+        if callback_options is not None:
+            override_budget = callback_options.pop("__sqreen_override_budget", None)
             if override_budget is not None:
                 return override_budget
         if self.runner is not None and self.performance_monitoring_enabled:
@@ -125,35 +157,6 @@ class BaseCallback(object):
             if budget is not None:
                 return budget - (self.storage.request_sq_time or 0)
 
-    def record_attack(self, infos=None, at=None):
-        """Record an attack."""
-        if at is None:
-            at = now()
-        payload = {
-            "infos": infos,
-            "rulespack_id": self.rulespack_id,
-            "rule_name": self.rule_name,
-            "beta": self.beta,
-            "block": self.block,
-            "test": self.test,
-            "attack_type": self.attack_type,
-            "time": at,
-        }
-        if "context" in self.payload_sections:
-            current_request = self.storage.get_current_request()
-            if current_request:
-                payload["backtrace"] = list(current_request.raw_caller)
-        LOGGER.debug("Observed attack %r", payload)
-        self.storage.observe("attacks", payload, self.payload_sections)
-
-    def record_observation(self, metric_name, key, value, at=None):
-        """Record a metric observation."""
-        if at is None:
-            at = now()
-        payload = (metric_name, at, key, value)
-        LOGGER.debug("Observed metric %r", payload)
-        self.storage.observe("observations", payload, report=False)
-
     def exception_infos(self, infos={}):
         """ Returns additional infos in case of exception
         """
@@ -185,7 +188,15 @@ class BaseCallback(object):
             "backtrace": bt,
         }
         LOGGER.debug("Observed exception %r", payload)
-        self.storage.observe("sqreen_exceptions", payload, self.payload_sections)
+        payload_sections = self.payload_sections
+        if payload_sections is None:
+            payload_sections = self.DEFAULT_EXCEPTION_SECTIONS
+        if "datadog-correlation-ids" in payload_sections:
+            trace_id, span_id = get_datadog_correlation_ids()
+            payload["datadog_trace_id"] = trace_id
+            payload["datadog_span_id"] = span_id
+        self.storage.observe("sqreen_exceptions", payload,
+                             payload_sections=payload_sections, report=True)
 
     def record_overtime(self, lifecycle, at=None):
         """Record an overtime execution."""
@@ -197,6 +208,29 @@ class BaseCallback(object):
             self.__class__.__name__, self.rule_name)
 
 
+def sampling_wrapper(wrapped, callback, sampler, lifecycle):
+    """ Wrapper that will execute the sampler to know if
+    the callback must be executed, otherwise return None.
+    """
+
+    @wraps(wrapped)
+    def wrapper(inst, args, kwargs, options):
+        decisions = options.get("sampling_decisions")
+        if decisions is None:
+            decisions = options["sampling_decisions"] = {}
+        # Has the sampler already been evaluated for this callback?
+        decision = decisions.get(callback.rule_name)
+        if decision is None:
+            # else shall we execute the callback?
+            decision = decisions[callback.rule_name] = sampler.should_sample(**options)
+        if decision:
+            # proceed with callback execution
+            return wrapped(inst, args, kwargs, options)
+
+    wrapper.__wrapped__ = wrapped
+    return wrapper
+
+
 def check_condition_wrapper(wrapped, callback, condition, lifecycle):
     """ Wrapper that will check lifecycle method pre-condition before
     calling it.
@@ -205,7 +239,7 @@ def check_condition_wrapper(wrapped, callback, condition, lifecycle):
     """
 
     @wraps(wrapped)
-    def wrapper(inst, args, kwargs, **options):
+    def wrapper(inst, args, kwargs, options):
         """ Wrapper around lifecycle method
         """
         storage = callback.storage
@@ -222,6 +256,7 @@ def check_condition_wrapper(wrapped, callback, condition, lifecycle):
             "data": callback.data,
             "rv": rv,
             "request_store": storage.get_request_store(),
+            "options": options,
         }
 
         # Check the pre condition
@@ -238,7 +273,7 @@ def check_condition_wrapper(wrapped, callback, condition, lifecycle):
             return None
 
         # Execute the hook otherwise with the original args
-        return wrapped(inst, args, kwargs, **options)
+        return wrapped(inst, args, kwargs, options)
 
     wrapper.__wrapped__ = wrapped
     return wrapper
@@ -249,7 +284,7 @@ def call_count_wrapper(wrapped, callback, lifecycle, observation_key):
     """
 
     @wraps(wrapped)
-    def wrapper(*args, **kwargs):
+    def wrapper(inst, args, kwargs, options):
         """ Record the number of calls for this callback lifecycle method.
         Buffer the number in the callback itself (self.call_counts) and record
         an observation every X times, X being the field call_count_interval of
@@ -258,14 +293,15 @@ def call_count_wrapper(wrapped, callback, lifecycle, observation_key):
         current_count = callback.call_counts[lifecycle]
 
         if current_count + 1 == callback.call_count_interval:
-            callback.record_observation(
-                "sqreen_call_counts", observation_key, callback.call_count_interval
+            # We do not need to pass by the request recorder here.
+            callback.runner.observation_queue.put(
+                ["sqreen_call_counts", options.get("at") or now(), observation_key, callback.call_count_interval]
             )
             callback.call_counts[lifecycle] = 0
         else:
             callback.call_counts[lifecycle] += 1
 
-        return wrapped(*args, **kwargs)
+        return wrapped(inst, args, kwargs, options)
 
     wrapper.__wrapped__ = wrapped
     return wrapper
@@ -280,11 +316,12 @@ def perf_monitoring_wrapper(wrapped, storage, name):
     """
 
     @wraps(wrapped)
-    def wrapper(*args, **kwargs):
-        """ Time the callback execution.
+    def wrapper(inst, args, kwargs, options):
+        """ Time the callback execution and update the at option entry.
         """
-        with storage.trace(name):
-            return wrapped(*args, **kwargs)
+        with storage.trace(name) as t:
+            options["at"] = t.start_at
+            return wrapped(inst, args, kwargs, options)
 
     wrapper.__wrapped__ = wrapped
     return wrapper
@@ -301,6 +338,16 @@ def debug_log_wrapper(wrapped, lifecycle, callback):
     return wrapper
 
 
+def action_processor_wrapper(wrapped, callback):
+
+    @wraps(wrapped)
+    def wrapper(inst, args, kwargs, options):
+        return callback.action_processor(wrapped(inst, args, kwargs, options), options)
+
+    wrapper.__wrapped__ = wrapped
+    return wrapper
+
+
 class RuleCallback(BaseCallback):
     """Rule callback attached to a hookpoint.
 
@@ -333,6 +380,7 @@ class RuleCallback(BaseCallback):
         priority=None,
         attack_render_info=None,
         attack_type=None,
+        sampling=None,
         storage=runtime,
     ):
         self.hook_module = hook_module
@@ -351,6 +399,7 @@ class RuleCallback(BaseCallback):
             payload_sections=payload_sections,
             priority=priority,
             attack_type=attack_type,
+            sampling=sampling,
             storage=storage,
         )
         self.attack_render_info = attack_render_info
@@ -368,8 +417,10 @@ class RuleCallback(BaseCallback):
         self.call_counts = {"pre": 0, "post": 0, "failing": 0}
 
         methods = self.lifecycle_methods
+        self._apply_action_processor(methods)
         self._apply_debug_log(methods)
         self._apply_conditions(methods)
+        self._apply_sampling(methods)
         self._apply_call_count(methods)
         self._apply_perf_monitoring(methods)
 
@@ -396,6 +447,7 @@ class RuleCallback(BaseCallback):
             payload_sections=rule_dict.get("payload"),
             priority=rule_dict.get("priority"),
             attack_type=rule_dict.get("attack_type"),
+            sampling=rule_dict.get("sampling"),
             storage=storage,
         )
 
@@ -435,11 +487,8 @@ class RuleCallback(BaseCallback):
 
     @property
     def lifecycle_methods(self):
-        return [
-            lifecycle
-            for lifecycle in LIFECYCLE_METHODS.values()
-            if hasattr(self, lifecycle)
-        ]
+        return [lifecycle for lifecycle in ("pre", "post", "failing",)
+                if hasattr(self, lifecycle)]
 
     def __repr__(self):
         return "%s(rule_name=%r, hook_module=%r, hook_name=%r, strategy=%r)" % (
@@ -465,6 +514,7 @@ class ReactiveRuleCallback(BaseCallback):
         payload_sections=None,
         priority=None,
         attack_type=None,
+        sampling=None,
         storage=runtime,
     ):
         super(ReactiveRuleCallback, self).__init__(
@@ -479,16 +529,19 @@ class ReactiveRuleCallback(BaseCallback):
             payload_sections=payload_sections,
             priority=priority,
             attack_type=attack_type,
+            sampling=sampling,
             storage=storage,
         )
         self.authorized_addresses = frozenset(authorized_addresses or [])
         self.batch_addresses = self.authorized_addresses
         self.group_addresses = frozenset()
 
+        self._apply_action_processor(["handler"])
         self._apply_debug_log(["handler"])
+        self._apply_sampling(["handler"])
         self._apply_perf_monitoring(["handler"])
 
-    def handler(self, instance, args, kwargs, **options):
+    def handler(self, instance, args, kwargs, options):
         """ Handler called when addresses are available.
         """
         raise NotImplementedError
@@ -510,5 +563,6 @@ class ReactiveRuleCallback(BaseCallback):
             payload_sections=rule_dict.get("payload"),
             priority=rule_dict.get("priority"),
             attack_type=rule_dict.get("attack_type"),
+            sampling=rule_dict.get("sampling"),
             storage=storage,
         )
