@@ -9,22 +9,23 @@
 import logging
 
 from ..rules import RuleCallback
+from ..rules_actions import RecordObservationMixin
 
 LOGGER = logging.getLogger(__name__)
 
 
-class CountHTTPCodesCBAioHTTP(RuleCallback):
+class CountHTTPCodesCBAioHTTP(RecordObservationMixin, RuleCallback):
 
     INTERRUPTIBLE = False
 
-    def post(self, instance, args, kwargs, result=None, **options):
+    def post(self, instance, args, kwargs, options):
         """Recover the status code and update the http_code metric."""
-        response = result
+        response = options.get("result")
         status_code = response.status
         self.record_observation("http_code", str(status_code), 1)
         return {}
 
-    def failing(self, instance, args, kwargs, **options):
+    def failing(self, instance, args, kwargs, options):
         """Recover the status code and update the http_code metric."""
         self.record_observation("http_code", "500", 1)
         return {}
