@@ -9,13 +9,14 @@
 
 from logging import getLogger
 
+from ..rules_actions import RecordObservationMixin
 from .matcher_callback import MatcherRule
 
 LOGGER = getLogger(__name__)
 
 
-class CrawlerUserAgentMatchesMetricsCB(MatcherRule):
-    def pre(self, instance, args, kwargs, **options):
+class CrawlerUserAgentMatchesMetricsCB(RecordObservationMixin, MatcherRule):
+    def pre(self, instance, args, kwargs, options):
         """ For each request, record an observation with the user_agent without
         case modification.
         """
