@@ -14,11 +14,11 @@ LOGGER = logging.getLogger(__name__)
 
 
 class HeadersInsertCBAioHTTP(BaseHeadersInsertCB):
-    def post(self, instance, args, kwargs, result=None, **options):
+    def post(self, instance, args, kwargs, options):
         """Insert custom headers."""
         try:
-            response = result
-            for header_name, header_value in self.headers.items():
+            response = options.get("result")
+            for header_name, header_value in self.headers.values():
                 response.headers[header_name] = header_value
         except Exception:
             LOGGER.warning("An error occurred", exc_info=True)
