@@ -15,5 +15,5 @@ LOGGER = logging.getLogger(__name__)
 
 class ReactiveLog(ReactiveRuleCallback):
 
-    def handler(self, instance, args, kwargs, result=None, **options):
+    def handler(self, instance, args, kwargs, options):
         LOGGER.info("Reactive data: %r", args[0])
