@@ -17,14 +17,14 @@ LOGGER = getLogger(__name__)
 
 class RecordRequestContextDjango(RecordRequestContext):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         self._store_request(DjangoRequest(args[0]))
 
-    def post(self, instance, args, kwargs, result=None, **options):
-        self.storage.store_response(DjangoResponse(result))
+    def post(self, instance, args, kwargs, options):
+        self.storage.store_response(DjangoResponse(options.get("result")))
         self._clear_request()
 
-    def failing(self, instance, args, kwargs, **options):
+    def failing(self, instance, args, kwargs, options):
         """ Post is always called in a Django Middleware, don't clean the
         request right now as it may be needed in a post callback
         """
