@@ -15,13 +15,13 @@ LOGGER = getLogger(__name__)
 
 
 class RecordRequestContextFlask(RecordRequestContext):
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         from flask import _request_ctx_stack
 
         reqctx = _request_ctx_stack.top
         if reqctx is not None:
             self._store_request(FlaskRequest(reqctx.request))
 
-    def post(self, instance, args, kwargs, result=None, **options):
-        self.storage.store_response(FlaskResponse(result))
+    def post(self, instance, args, kwargs, options):
+        self.storage.store_response(FlaskResponse(options.get("result")))
         self._clear_request()
