@@ -19,7 +19,7 @@ class RecordResponse(RuleCallback):
 
     INTERRUPTIBLE = False
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         """ Record the WSGI response if empty.
         """
         current_request = self.storage.get_current_request()
