@@ -9,6 +9,7 @@
 import sys
 from logging import getLogger
 
+from ..rules_actions import RecordAttackMixin
 from ..utils import STRING_CLASS, UNICODE_CLASS, is_string
 from .regexp_rule import RegexpRule
 
@@ -23,12 +24,13 @@ else:
 LOGGER = getLogger(__name__)
 
 
-class ReflectedXSSCB(RegexpRule):
-    def post(self, instance, args, kwargs, result=None, **options):
+class ReflectedXSSCB(RecordAttackMixin, RegexpRule):
+    def post(self, instance, args, kwargs, options):
         """ Check if a template node returns a content that is in the
         query parameters
         """
         request = self.storage.get_current_request()
+        result = options.get("result")
 
         if not request:
             LOGGER.warning("No request was recorded abort")
