@@ -9,14 +9,15 @@
 import os
 from logging import getLogger
 
+from ..rules_actions import RecordAttackMixin
 from ..utils import Mapping, is_string
 from .regexp_rule import RegexpRule
 
 LOGGER = getLogger(__name__)
 
 
-class ShellEnvCB(RegexpRule):
-    def pre(self, instance, args, kwargs, **options):
+class ShellEnvCB(RecordAttackMixin, RegexpRule):
+    def pre(self, instance, args, kwargs, options):
         """
         Examine environment variables to find shellshock injections.
         Works if env is passed in positional or keyword argument.
