@@ -17,7 +17,7 @@ LOGGER = getLogger(__name__)
 class SqreenErrorPageAioHTTP(BaseSqreenErrorPage):
     """Custom error page for aiohttp."""
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
 
         exc = kwargs.get("exc") or args[2]
         ret = self.handle_exception(exc)
