@@ -12,7 +12,7 @@ from .sqreen_error_page import BaseSqreenErrorPage
 
 class SqreenErrorPageDjango(BaseSqreenErrorPage):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
 
         exc_info = args[2]
         if not exc_info:
