@@ -12,7 +12,7 @@ from .sqreen_error_page import BaseSqreenErrorPage
 
 class SqreenErrorPagePyramid(BaseSqreenErrorPage):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
 
         context = args[2]
         ret = self.handle_exception(context)
