@@ -8,14 +8,15 @@
 """
 from logging import getLogger
 
+from ..rules_actions import RecordAttackMixin
 from .regexp_rule import RegexpRule
 
 LOGGER = getLogger(__name__)
 
 
-class UserAgentMatchesCBFramework(RegexpRule):
+class UserAgentMatchesCBFramework(RecordAttackMixin, RegexpRule):
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
 
         request = self.storage.get_current_request()
 
