@@ -13,6 +13,7 @@ from weakref import WeakKeyDictionary
 from ..binding_accessor import BindingAccessor
 from ..exceptions import InvalidArgument, SqreenException
 from ..rules import RuleCallback
+from ..rules_actions import RecordAttackMixin, RecordObservationMixin
 from ..runner import BaseRunner
 from ..runtime_infos import get_agent_type
 from ..samplers import ScopedTracingSampler
@@ -24,7 +25,7 @@ else:
     from collections import Mapping
 
 try:
-    from sq_native import waf, input as waf_input  # type: ignore
+    from sq_native import input as waf_input, waf  # type: ignore
 except Exception:
     waf = None  # pragma: no cover
 
@@ -83,7 +84,7 @@ class WAFInstances:
             return instance
 
 
-class WAFCBMixin(object):
+class WAFCBMixin(RecordAttackMixin, RecordObservationMixin):
 
     SUPPORTS_BUDGET = True
 
@@ -187,7 +188,7 @@ class WAFCB(WAFCBMixin, RuleCallback):
                 bas[item] = (BindingAccessor(item), None)
         self.binding_accessors = bas
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         request = self.storage.get_current_request()
         if request is None:
             return
