@@ -10,7 +10,6 @@ import sys
 from io import BytesIO
 from logging import getLogger
 
-from ..constants import ACTIONS
 from ..rules import RuleCallback
 
 LOGGER = getLogger(__name__)
@@ -99,10 +98,13 @@ class WSGIInputPreview(RuleCallback):
 
     INTERRUPTIBLE = False
 
-    def pre(self, instance, args, kwargs, **options):
+    def pre(self, instance, args, kwargs, options):
         """ Replace the WSGI input stream with our wrapper to get a
         preview of the body.
         """
+        # Do not copy the environ but mutate it because some
+        # webservers keep a reference on the original environ
+        # and do not except the reference to change.
         environ = args[0]
         input = environ.get("wsgi.input")
         current_request = self.storage.get_current_request()
@@ -112,9 +114,10 @@ class WSGIInputPreview(RuleCallback):
             input_wrapper = WSGIInputWrapper(
                 self, input, current_request.content_length,
                 max_preview_length)
-            environ = dict(environ.items())
             environ["wsgi.input"] = input_wrapper
+        # Return a modify_args because we should replace
+        # the environ, but the mutatation is enough
         return {
-            "status": ACTIONS["MODIFY_ARGS"],
+            "status": "modify_args",
             "args": ([environ], kwargs)
         }
