@@ -19,6 +19,7 @@ from logging import getLogger
 from threading import Event, Thread
 
 from .__about__ import __version__
+from ._vendors.sqreen_security_signal_sdk.sender import AuthenticationFailed
 from ._vendors.urllib3 import PoolManager, ProxyManager
 from ._vendors.urllib3.exceptions import HTTPError  # type: ignore
 from ._vendors.urllib3.util import Timeout
@@ -58,6 +59,7 @@ from .utils import (
     HAS_TYPING,
     configure_newrelics_ignore_exception,
     configure_raven_breadcrumbs,
+    configure_sentry_ignore_errors,
 )
 
 if sys.version_info[0] < 3:
@@ -66,11 +68,10 @@ if sys.version_info[0] < 3:
     from ._vendors.concurrent.futures import ThreadPoolExecutor, TimeoutError
 else:
     import urllib.parse as urlparse
-
     from concurrent.futures import ThreadPoolExecutor, TimeoutError
 
 if HAS_TYPING:
-    from typing import Sequence, Optional
+    from typing import Optional, Sequence
 
 RUNNER_THREAD = None
 RUNNER_THREAD_RESTARTING = False
@@ -150,6 +151,8 @@ def get_app_name():
     config_app_name = CONFIG["APP_NAME"]
     if config_app_name is not None:
         app_name = config_app_name
+    elif CONFIG["SERVERLESS_DETECTION"] and "AWS_LAMBDA_FUNCTION_NAME" in os.environ:
+        app_name = os.environ["AWS_LAMBDA_FUNCTION_NAME"]
     else:
         app_name = "python-application"
 
@@ -167,7 +170,7 @@ def get_app_name():
     return app_name
 
 
-def get_session(url, token, app_name=None, proxy_url=None, use_legacy_url=False):
+def get_session(url, token, app_name=None, proxy_url=None, use_legacy_url=False, ingestion_url=None):
     """ Create a connection with the endpoint URL and a session. Returns only
     the session.
     """
@@ -178,7 +181,7 @@ def get_session(url, token, app_name=None, proxy_url=None, use_legacy_url=False)
         " using proxy %s" % proxy_url if proxy_url else "",
     )
     con = Urllib3Connection(url, proxy_url=proxy_url, use_legacy_url=use_legacy_url)
-    session = Session(con, token, app_name)
+    session = Session(con, token, app_name, ingestion_url=ingestion_url)
     return session
 
 
@@ -292,8 +295,9 @@ def start():
         post_app_sqreen_exception(RemoteException.from_exc_info())
         return
 
-    # Configure raven breadcrumbs at start
+    # Configure raven breadcrumbs and sentry SDK at start
     configure_raven_breadcrumbs()
+    configure_sentry_ignore_errors()
 
     # Ignore Sqreen Exception from APM NewRelics
     configure_newrelics_ignore_exception()
@@ -306,13 +310,17 @@ def start():
             RUNNER_THREAD = RunnerThread(runtime_infos)
             RUNNER_THREAD.start()
 
-            done = RUNNER_THREAD.unblock_main_thread.wait(CONFIG["INSTRUMENTATION_TIMEOUT"])
+            timeout = CONFIG["INSTRUMENTATION_TIMEOUT"]
+            if RUNNER_THREAD.serverless:
+                timeout = min(timeout, CONFIG["SERVERLESS_INSTRUMENTATION_TIMEOUT"])
+
+            done = RUNNER_THREAD.unblock_main_thread.wait(timeout)
             if done is not True:
                 LOGGER.critical(
                     "Sqreen could not instrument your application in less than %ds.\n"
-                    "Please check your account is not expired and your application has network connectivity.\n"
+                    "Please retry in a few seconds.\n"
                     "You can also check Sqreen status on %s.",
-                    CONFIG["INSTRUMENTATION_TIMEOUT"], STATUS_URL)
+                    timeout, STATUS_URL)
 
         except Exception:
             LOGGER.critical(
@@ -434,21 +442,26 @@ class RunnerThread(Thread):
         super(RunnerThread, self).__init__()
         self.daemon = True
         self.queue = queue or CappedQueue(CONFIG["MAX_QUEUE_LENGTH"])
-        self.observations_queue = observations_queue or CappedQueue(CONFIG["MAX_OBS_QUEUE_LENGTH"], name="observations queue")
+        self.observations_queue = observations_queue or CappedQueue(CONFIG["MAX_OBS_QUEUE_LENGTH"], name="observations_queue")
         self.unblock_main_thread = Event()
         self.runtime_infos = runtime_infos
-        self.instrumentation = instrumentation or Instrumentation(
-            self.observations_queue, self.queue, before_hook_point
-        )
+        self.instrumentation = instrumentation or Instrumentation(before_hook_point)
         self.settings = settings or RunnerSettings()
         if CONFIG["BETA_STATIC_INSTRUMENTATION"]:
             interface_manager = interface_manager or InterfaceManager()
         self.interface_manager = interface_manager
-        self.reactive = reactive or Reactive(self.observations_queue, self.queue)
+        self.reactive = reactive or Reactive()
         self.runner = None
         self.started = False
         self.pid = None
 
+    @property
+    def serverless(self):
+        if not CONFIG["SERVERLESS_DETECTION"]:
+            return False
+        aws_lambda = self.runtime_infos.get("various_infos", {}).get("aws_lambda", {})
+        return aws_lambda.get("sqreen_extension", False)
+
     @property
     def stacktrace(self):
         """Get the runner thread stack trace if it exists.
@@ -494,20 +507,33 @@ class RunnerThread(Thread):
 
                 LOGGER.warning("Using token %r and application name %r", token, app_name)
 
-                if not CONFIG["NO_SNIFF_DOMAINS"]:
-                    # Detect if the configured URLs are reachable.
-                    use_legacy_url = not sqreen_compatible_backends(
-                        [CONFIG["URL"], CONFIG["INGESTION_URL"]], proxy_url=get_proxy_url())
+                if not self.serverless:
+                    if not CONFIG["NO_SNIFF_DOMAINS"]:
+                        # Detect if the configured URLs are reachable.
+                        use_legacy_url = not sqreen_compatible_backends(
+                            [CONFIG["URL"], CONFIG["INGESTION_URL"]], proxy_url=get_proxy_url())
+                    else:
+                        # Force use of the configured URLs.
+                        use_legacy_url = False
+
+                    # Initiate HTTP connection to the backend and the session, fallback to the legacy URL
+                    # if the new URLs are not available.
+                    if use_legacy_url:
+                        LOGGER.info("Using legacy URLs for backend communication")
+                        session_url = ingestion_url = CONFIG["LEGACY_URL"]
+                    else:
+                        LOGGER.info("Using new URLs for backend communication")
+                        session_url, ingestion_url = CONFIG["URL"], CONFIG["INGESTION_URL"]
+                    proxy_url = get_proxy_url()
                 else:
-                    # Force use of the configured URLs.
+                    LOGGER.info("Using serverless extension for backend communication")
+                    session_url = ingestion_url = CONFIG["SERVERLESS_EXTENSION_URL"]
+                    proxy_url = None
                     use_legacy_url = False
 
-                # Initiate HTTP connection to the backend and the session, fallback to the legacy URL
-                # if the new URLs are not available.
-                session_url = CONFIG["LEGACY_URL"] if use_legacy_url else CONFIG["URL"]
                 session = get_session(
-                    session_url, token, app_name, proxy_url=get_proxy_url(),
-                    use_legacy_url=use_legacy_url
+                    session_url, token, app_name, proxy_url=proxy_url, use_legacy_url=use_legacy_url,
+                    ingestion_url=ingestion_url
                 )
 
                 runtime_infos = dict(self.runtime_infos)
@@ -533,13 +559,13 @@ class RunnerThread(Thread):
                         "Sorry but Sqreen's backend appears to be unavailable.\n"
                         "Your application is NOT currently protected by Sqreen.\n"
                         "\n"
-                        "Please check http://sqreen.status.io for outage.\n"
+                        "Please check %s for outage.\n"
                         "\n"
-                        "Otherwise, back.sqreen.io may be blocked by your server's network.\n"
+                        "Otherwise, back.sqreen.com may be blocked by your server's network.\n"
                         "If necessary, you may tell Sqreen to connect over a proxy.\n"
                         "Please visit https://docs.sqreen.com/python/configuration/ for more details."
                     )
-                    LOGGER.critical(msg)
+                    LOGGER.critical(msg, STATUS_URL)
                     self.unblock_main_thread.set()
                     return
 
@@ -565,21 +591,17 @@ class RunnerThread(Thread):
                 # Process the initial commands
                 process_initial_commands(login_result, self.runner)
 
-                retry = 3
+                self.unblock_main_thread.set()
+
+                # No background thread for the serverless mode
+                if self.serverless:
+                    return
+
                 while True:
                     try:
-                        # if we did not receive instrumentation_enable, try
-                        # to get it faster before the MainThread stops waiting.
-                        while not self.instrumentation.enabled and retry > 0:
-                            retry -= 1
-                            self._random_sleep(2.0)
-                            self.runner.do_heartbeat()
-                        self.unblock_main_thread.set()
                         self.runner.run()
                     except HTTPError:
-                        LOGGER.exception(
-                            "An HTTP error occurred, restarting runner"
-                        )
+                        LOGGER.debug("An HTTP error occurred, restarting runner", exc_info=True)
                         try:
                             session.post_sqreen_exception(
                                 RemoteException.from_exc_info()
@@ -590,11 +612,11 @@ class RunnerThread(Thread):
                     else:
                         # If the runner exits normally, return.
                         return
-            except InvalidSession:
-                LOGGER.info("Relaunching the session because it was dropped by the backend.")
+            except (AuthenticationFailed, InvalidSession):
+                LOGGER.debug("Relaunching the session because it was dropped by the backend.")
                 self._random_sleep()
             except Exception as exc:
-                LOGGER.exception("An unexpected exception occurred")
+                LOGGER.debug("An unexpected exception occurred", exc_info=True)
 
                 callback_payload = getattr(exc, "callback_payload", None)
                 exception_payload = getattr(exc, "exception_payload", None)
@@ -610,19 +632,19 @@ class RunnerThread(Thread):
                         session.post_sqreen_exception(remote_exception)
                         session.logout()
                     except HTTPError:
-                        LOGGER.exception("An HTTP error occurred while logout")
+                        LOGGER.debug("An HTTP error occurred while logout", exc_info=True)
                     except Exception:
-                        LOGGER.exception("Exception while logout")
+                        LOGGER.debug("Exception while logout", exc_info=True)
                         return
                 else:
                     post_app_sqreen_exception(remote_exception)
 
                 try:
-                    self.instrumentation.deinstrument_all()
+                    self.instrumentation.settings.remove_callbacks()
                 except Exception:
                     # We did not managed to remove instrumentation, state is unclear:
                     # terminate thread
-                    LOGGER.exception("Exception while trying to clean-up")
+                    LOGGER.debug("Exception while trying to clean-up", exc_info=True)
                     return
 
                 self._random_sleep()
