@@ -10,7 +10,7 @@ import datetime
 import json
 import sys
 import types
-from collections import deque
+from collections import defaultdict, deque
 from inspect import isclass
 from itertools import islice
 from logging import getLogger
@@ -21,6 +21,11 @@ from ._vendors.ipaddress import (  # type: ignore
     ip_network as _ip_network,
 )
 
+if sys.version_info[0] >= 3:
+    from urllib.parse import unquote_to_bytes
+else:
+    from urllib import unquote as unquote_to_bytes
+
 if sys.version_info >= (3, 5):
     from typing import (
         Any,
@@ -174,6 +179,29 @@ def to_unicode_safe(value):  # type: (Any) -> Optional[str]
     return value
 
 
+# Sentry SDK
+def configure_sentry_ignore_errors():
+    """ Ignore all logging messages from sqreen.* loggers, effectively
+    disabling sentry_sdk to log sqreen log messages as breadcrumbs
+    """
+    try:
+        from sentry_sdk.scope import add_global_event_processor
+    except ImportError:
+        return
+
+    try:
+        @add_global_event_processor
+        def ignore_sqreen_errors(event, hint):
+            try:
+                if event.get("logger", "").startswith("sqreen"):
+                    return None
+                return event
+            except Exception:
+                pass
+    except Exception:
+        LOGGER.warning("Error while configuring sentry_sdk", exc_info=True)
+
+
 ###
 # Raven configuration
 ###
@@ -566,3 +594,25 @@ class cached_property(property):
                 return None
             obj.__dict__[self.__name__] = value
         return value
+
+
+def get_datadog_correlation_ids():
+    """Return the trace ID and span ID from dd-trace.
+    """
+    try:
+        from ddtrace import helpers
+
+        return helpers.get_correlation_ids()
+    except Exception:
+        return None, None
+
+
+def parse_form_params(qs):
+    """Return a dict of form data after Basic HTTP form parsing."""
+    qs = to_unicode_safe(qs).replace("+", " ")
+    params = defaultdict(list)
+    for item in qs.split("&"):
+        k, equal, v = item.partition("=")
+        if equal:
+            params[to_unicode_safe(unquote_to_bytes(k))].append(to_unicode_safe(unquote_to_bytes(v)))
+    return dict(params)
